#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import sys, sha, time, urlparse
from ConfigParser import ConfigParser

import ZSI.TC
from ZSI.wstools.Namespaces import WSU

import pyGridWare.generated.stubs.Counter_services as COUNTER
from pyGridWare.generated.types.ResourceLifetime_types import \
    www_ibm_com_xmlns_stdwip_web_services_WS_ResourceLifetime as LIFETIME

from pyGridWare.generated.types.ResourceProperties_types import \
    www_ibm_com_xmlns_stdwip_web_services_WS_ResourceProperties as PROPERTIES

from pyGridWare.generated.types.Addressing_types import *
from pyGridWare.security.gss.GssSignatureHandler import GssSignatureHandler
from pyGridWare.security.gss.Authenticate import GssAuthenticationHandler

# Add WSU:Id attribute typecode to signal this node is to be signed.
COUNTER.ns7.createCounter_Dec.attribute_typecode_dict = {(WSU.BASE,"Id"):ZSI.TC.String()}
COUNTER.ns7.add_Dec.attribute_typecode_dict = {(WSU.BASE,"Id"):ZSI.TC.String()}
LIFETIME.Destroy_Dec.attribute_typecode_dict = {(WSU.BASE,"Id"):ZSI.TC.String()}
LIFETIME.SetTerminationTime_Dec.attribute_typecode_dict = {(WSU.BASE,"Id"):ZSI.TC.String()}
PROPERTIES.GetResourceProperty_Dec.attribute_typecode_dict = {(WSU.BASE,"Id"):ZSI.TC.String()}

CONFIG_FILE = 'config.txt'
SEC_PATHS = 'Paths'
SEC_LOCATION = 'Location'

class SignInt(int):
    def __init__(self, value):
        self._attrs = {}
        self._attrs[(WSU.BASE,"Id")] = "digestSource"
        int.__init__(self,value) 

class QualifiedName(tuple):
    def __init__(self, value):
        self._attrs = {}
        self._attrs[(WSU.BASE,"Id")] = "digestSource"
        tuple.__init__(self,value)


"""
Counter
    add
    createCounter
"""
def add(port, value=999):
    print "----- add ----- "
    request = SignInt(value)
    msg = port.add(request)
    print "RESPONSE: ", msg

def create(port):
    """
  <createCounterResponse xmlns="http://counter.com">
   <wsa:EndpointReference>
    <wsa:Address>http://192.168.1.104:8080/wsrf/services/CounterService</wsa:Address>
    <wsa:ReferenceProperties>
     <CounterKey>28939486</CounterKey>
    </wsa:ReferenceProperties>
   </wsa:EndpointReference>
  </createCounterResponse>
    """
    request = COUNTER.CreateCounterRequest()
    request._attrs = {}
    request._attrs[(WSU.BASE,"Id")] = "digestSource"
    msg = port.createCounter(request)
    print "----- create ----- "
    print "\tAddress -- ", msg._EndpointReference._Address
    print "\tRefProp -- ", msg._EndpointReference._ReferenceProperties
    print "\t\tany -- ", msg._EndpointReference._ReferenceProperties._any
    return msg

def add(port, value=999):
    print "----- add ----- "
    request = SignInt(value)
    msg = port.add(request)
    print "RESPONSE: ", msg

"""
ResourceLifetime
    Destroy
    GetCurrentMessage
    SetTerminationTime
"""
def Destroy(port):
    print "----- destroy ----- "
    request = COUNTER.DestroyRequest()
    request._attrs = {}
    request._attrs[(WSU.BASE,"Id")] = "digestSource"
    msg = port.Destroy(request)
    print "RESPONSE: ", msg

def SetTerminationTime(port, interval):
    print "----- SetTerminationTime ----- "
    request = COUNTER.SetTerminationTimeRequest()
    request._RequestedTerminationTime = time.gmtime(time.time()+interval)
    request._attrs = {}
    request._attrs[(WSU.BASE,"Id")] = "digestSource"
    msg = port.SetTerminationTime(request)
    print "RESPONSE: ", msg

"""
ResourceProperties
    GetResourceProperty
"""
def GetResourceProperty(port):
    print "----- GetResourceProperty ----- "
    request = QualifiedName(("http://counter.com", "Value"))
    request._attrs = {}
    request._attrs[(WSU.BASE,"Id")] = "digestSource"
    msg = port.GetResourceProperty(request)
    print "RESPONSE: ", msg



"""
NotificationSubscriber
    Subscribe
"""
def Subscribe(port):
    pass


def counter(url, path, KeyName):
    kw = {'tracefile':sys.stdout, 'url':path}
    scheme,netloc,path,params,query,fragment = urlparse.urlparse(url)
    locator = COUNTER.CounterServiceLocator()

    sig_handler = GssSignatureHandler()
    sig_handler.setContextID(KeyName)
    kw['sig_handler'] = sig_handler

    port = locator.getCounterPortType(portAddress=url, **kw)
    response = create(port)
    epr = response._EndpointReference

    port = locator.getCounterPortType(portAddress=url, endPointReference=epr, **kw)
    add(port, 11)
    #Destroy(port)
    SetTerminationTime(port, 10)
    add(port, 13)
    GetResourceProperty(port)
    #Destroy(port)
    time.sleep(15)
    add(port, 15)

    
def main():
    cp = ConfigParser()
    cp.read(CONFIG_FILE)

    path = cp.get(SEC_PATHS, 'AuthService')
    host = cp.get(SEC_LOCATION, 'host')
    port = cp.getint(SEC_LOCATION, 'port')

    netloc = '%s:%d'%(host,port)
    urlTuple = ('http',netloc,path,'','','')
    url = urlparse.urlunparse(urlTuple)

    gssAuth = GssAuthenticationHandler()
    KeyName = gssAuth.authenticate(url)

    path = cp.get(SEC_PATHS, 'SecureCounterService')
    urlTuple = ('http',netloc,path,'','','')
    url = urlparse.urlunparse(urlTuple)
    counter(url, path, KeyName)


if __name__ == '__main__':
    main()

