#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import sys, urlparse

import ZSI.TC
from ZSI.wstools.Namespaces import SOAP
from ZSI.utility import ElementProxy

import pyGridWare.generated.stubs.Authenticator_services as AUTH
from pyGridWare.generated.types.Addressing_types import *
from pyGridWare.security.gss.GssSecureConversation import GssSecurityContext
from pyGridWare.security.gss.GssSignatureHandler import GssSignatureHandler


class Holder: pass

# Add soapenc:actor attribute typecode
AUTH.schemas_xmlsoap_org_ws_2002_04_secext.Security_Dec.attribute_typecode_dict = {(SOAP.ENV,'actor'):ZSI.TC.String()}

class GssAuthenticationHandler:
    mechanism = "http://www.globus.org/2002/04/xmlenc#gssapi-gsi"
    namespaceURI = "http://wsrf.globus.org/security/authentication/secureconv/types"

    #def setDelegation(self):
    #    self._context.setDelegation()

    def getMechanism(cls):
        return cls.mechanism
    getMechanism = classmethod(getMechanism)

    def getNamespace(cls):
        return cls.namespaceURI
    getNamespace = classmethod(getNamespace)

    def authenticate(self, url, delegation=True, tracefile=None):
        context = GssSecurityContext()
        mechanism = GssAuthenticationHandler.getMechanism()
        namespaceURI = GssAuthenticationHandler.getNamespace()
        mechanism_attr_key = (namespaceURI,"mechanism-type")
        continue_attr_key = (namespaceURI,"continue-needed")

        if delegation is True:
            context.setDelegation()

        # Initialize Port instance
        locator = AUTH.AuthenticationServiceLocator()
        scheme,netloc,path,params,query,fragment = urlparse.urlparse(url)
        kw = {'url':path}
        if tracefile is not None:
            kw['tracefile'] = tracefile
        port = locator.getSecureContextEstablishmentPortType(portAddress=url, **kw)

        # Construct initTokenExchangeRequest
        token = context.initSecurityContext()
        initMsgIn = AUTH.InitTokenExchangeRequest()
        initMsgIn._initContextToken = Holder()
        initMsgIn._initContextToken._attrs = \
            {mechanism_attr_key:mechanism, continue_attr_key:"true"}
        initMsgIn._initContextToken._context_id = context.hashCode()
        initMsgIn._initContextToken._base64Token = token

        # Read initTokenExchangeResponse
        initMsgOut = port.initTokenExchange(initMsgIn)
        base64Token = initMsgOut._contextTokenOut._base64Token
        context_id = initMsgOut._contextTokenOut._context_id
        context.setContextID(context_id)
        continue_needed = initMsgOut._contextTokenOut._attrs[continue_attr_key]

        while continue_needed == 'true' or continue_needed == '1':
            token = context.initSecurityContext(inToken=base64Token)

            # Construct continueTokenExchangeRequest
            cMsgIn = AUTH.ContinueTokenExchangeRequest()
            cMsgIn._contextToken = Holder()
            cMsgIn._contextToken._context_id = context_id
            cMsgIn._contextToken._base64Token = token
            cMsgIn._contextToken._attrs = {continue_attr_key:continue_needed}
            try:
                cMsgOut = port.continueTokenExchange(cMsgIn)
            except Exception, ex:
                raise
            else:
                # Read continueTokenExchangeResponse
                continue_needed = cMsgOut._contextTokenOut._attrs[continue_attr_key]
                base64Token = cMsgOut._contextTokenOut._base64Token
                context_id = cMsgOut._contextTokenOut._context_id

        # Add GssContext to GssContextHome
        contextID = context.getContextID()
        home = GssSecurityContext.getResourceHome()
        home.create(context, contextID)
        return contextID


