############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import sys, types, string

from Ft.Lib import Uuid
from pyGlobus.security import GSSCred, GSSContext, GSSContextException
from pyGlobus.security import GSSName, GSSMechs, GSSUsage, GSSBuffer
from pyGlobus.security import ContextRequests

from pyGridWare import getLogger
from pyGridWare.resource.ResourceHome import SimpleResourceHome

logger = getLogger(__name__)


class SecurityManager: pass
class GSSManager(SecurityManager):
    """
    """
    def createCredential(self):
        cred = GSSCred()
        name, mechs, usage  = GSSName(), GSSMechs(), GSSUsage()
        usage.set_usage_initiate()
        cred.acquire_cred(name, mechs, usage)
        return cred

    def createContext(self, cred):
        lifetime, credName = cred.inquire_cred()
        context = GSSContext()
        requests = ContextRequests()
        return context, requests, lifetime, credName
           

#class PersistentResource: pass
class GssSecurityContextHome(SimpleResourceHome):
    """
    """
    pass
"""
    def __init__(self):
       self.context_d = {}

    def create(self, context, key):
       self.context_d[key] = context

    def get(self, key):
       if not self.context_d.has_key(key):
           raise RuntimeError, 'no context id(%s) in home %s' %(key,self.context_d.keys())
       return self.context_d[key]
"""

class GssSecurityContext:
    """Associates a GSSContext with a context Id. It is used for GSI secure converstation.
    """
    home = GssSecurityContextHome()

    def __init__(self):
        manager = GSSManager()
        self._contextID = None
        self._cred = manager.createCredential()
        self._GSSContext, self._requests, lifetime, self._credName = \
            manager.createContext(self._cred)

    def getContext(self):
        """org.ietf.jgss.GSSContext getContext()
        """
        return self._GSSContext

    def getContextID(self):
        """java.lang.String getContextId()
        """
        return self._contextID

    def getDelegatedCredential(self):
        """org.ietf.jgss.GSSCredential getDelegatedCredential() 
        """
        self._cred

    def getResourceHome(cls):
        return cls.home 
    getResourceHome = classmethod(getResourceHome)
   
    def setCredential(self, cred):
        self._cred = cred

    def setContext(self, context):
        self._GSSContext = context

    def setContextID(self, contextID):
        self._contextID = contextID

    def setDelegation(self):
        self._requests.set_delegation()

    def setEncryption(self):
        self._requests.set_confidentiality()

    def hashCode(self):
        """init context-id
        """
        hcode = Uuid.UuidAsString(Uuid.GenerateUuid())
        tmpList = hcode.split('-')
        tmpList[3] = '0' * 8
        tmpList[4] = '0' * 12
        return(string.join(tmpList, '-'))

    def acceptSecurityContext(self, inToken):
        major,minor,outToken = \
            self._GSSContext.accept_context(\
                acceptor_cred=self._cred, 
                inputTokenString=inToken)
        return outToken

    def initSecurityContext(self, inToken=None):
        """XXX pyGlobus throws a segmentation Fault if inToken=None

           The ContextRequests object needs to be the last argument to the
           init_context calls that you make.
        """
        if inToken:
            major,minor,outToken = \
                self._GSSContext.init_context(\
                    init_cred=self._cred, 
                    target_name=self._credName, 
                    inputTokenString=inToken, 
                    requests=self._requests)
        else:
            major,minor,outToken = \
                self._GSSContext.init_context(\
                    init_cred=self._cred, 
                    target_name=self._credName,
                    inputTokenString="",
                    requests=self._requests)
        return outToken

    def getMIC(self, signedInfoC14n):
        signatureValue = self._GSSContext.get_mic(signedInfoC14n)
        return signatureValue

    def verifyMIC(self, signedInfoC14n, signatureValue):
        return self._GSSContext.verify_mic(signedInfoC14n, signatureValue)

    def wrap(self, messageStr):
        return self._GSSContext.wrap(messageStr)

    def unwrap(self, encryptedStr):
        return self._GSSContext.unwrap(encryptedStr)



